#include "system_monitor.hpp"


using namespace std;
using namespace mysqlpp;
using boost::property_tree::ptree;

system_monitor::system_monitor(DDSDomainParticipant *participant) : participant_(participant)
{

    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/system_monitor_subscriber_qos.xml";
//    cout << qos_file_ << endl;

	/* To customize the subscriber QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	subscriber_ = participant_->create_subscriber(
			DDS_SUBSCRIBER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);
	if (subscriber_ == NULL) {
		syslog(LOG_ERR, "system_monitor, create_subscriber error");
	}

	/* Register the type before creating the topic */
	type_name_ = sysmon_statusTypeSupport::get_type_name();
	retcode_ = sysmon_statusTypeSupport::register_type(
			participant_, type_name_);
	if (retcode_ != DDS_RETCODE_OK) {
		syslog(LOG_ERR, "system_monitor, register_type error %d", retcode_);
	}

	/* To customize the topic QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	topic_ = participant_->create_topic(
			"sysmon_status",
			type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
			DDS_STATUS_MASK_NONE);
	if (topic_ == NULL) {
		syslog(LOG_ERR, "system_monitor, create_topic error");
	}


}


void system_monitor::receive_information(void)
{

	/* Create a data reader listener */
	reader_listener_ = new sysmon_statusListener();

	/* To customize the data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	reader_ = subscriber_->create_datareader(
			topic_, DDS_DATAREADER_QOS_DEFAULT, reader_listener_,
			DDS_STATUS_MASK_ALL);
	if (reader_ == NULL) {
		syslog(LOG_ERR, "system_monitor, create_datareader error");
		delete reader_listener_;
	}

}


sysmon_statusListener::sysmon_statusListener()
{
    string sysconf_dir(SYSCONFDIR);
    string filename(sysconf_dir + "cavecanem_subscriber.xml");
    
	try {
		ptree pt;
		read_xml(filename,pt);
    	//Reading database data
        database_server_ = pt.get<string>("database.server");
    	database_name_ = pt.get<string>("database.name");
    	database_user_ = pt.get<string>("database.user");
    	database_password_ = pt.get<string>("database.password");
    	rows_ = pt.get<int>("plugins.system_monitor_plugin.max_rows");
    }
    
	catch(std::exception &e) {
		syslog(LOG_ERR,"system_monitor %s",e.what());
		rows_ = 10;     
	}

	conn_.connect(
	    database_name_.c_str(), 
	    database_server_.c_str(), 
	    database_user_.c_str(),
	    database_password_.c_str());
	    
	mysqlpp::Query query = conn_.query();

    query << "CREATE TABLE IF NOT EXISTS hosts (" << 
             "hostname VARCHAR(30) NOT NULL PRIMARY KEY," <<
             "sysname VARCHAR(30)," <<
             "release_name VARCHAR(30)," <<
             "machine VARCHAR(30))";
    query.execute();

	query.exec("DROP TABLE IF EXISTS system_monitor");

	query << "CREATE TABLE system_monitor (" <<
			"rrd_sm_seq_id INT UNSIGNED NOT NULL DEFAULT 0," <<
			"hostname VARCHAR(30) NOT NULL," <<
			"ts TIMESTAMP NOT NULL," <<
			"cpu_temp REAL," <<
			"load_one REAL," <<
			"load_five REAL," <<
			"load_fifteen REAL," <<
			"cpu_user REAL," <<
			"cpu_system REAL," <<
			"cpu_idle REAL," <<
			"mem_total REAL," << \
			"mem_used REAL," <<
			"mem_free REAL," <<
			"swap_total REAL," <<
			"swap_used REAL," <<
			"swap_free REAL," <<
			"PRIMARY KEY(rrd_sm_seq_id,hostname)," <<
			"FOREIGN KEY(hostname) REFERENCES hosts(hostname))";
	query.execute();
}


void sysmon_statusListener::on_data_available(DDSDataReader* reader)
{
	sysmon_statusDataReader *sysmon_status_reader = NULL;
	sysmon_statusSeq data_seq;
	DDS_SampleInfoSeq info_seq;
	DDS_ReturnCode_t retcode;
	int i;

	sysmon_status_reader = sysmon_statusDataReader::narrow(reader);
	if (sysmon_status_reader == NULL) {
		syslog(LOG_ERR,"system_monitor, DataReader narrow error");
		return;
	}

	retcode = sysmon_status_reader->take(
			data_seq, info_seq, DDS_LENGTH_UNLIMITED,
			DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

	if (retcode == DDS_RETCODE_NO_DATA) {
		return;
	} else if (retcode != DDS_RETCODE_OK) {
		syslog(LOG_ERR, "system_monitor, take error %d", retcode);
		return;
	}

	string query;
	for (i = 0; i < data_seq.length(); ++i) {
		if (info_seq[i].valid_data) {

			mysqlpp::Query query = conn_.query();
			
            map<string,string>::iterator hostinfo_map_it;
            hostinfo_map_it = hostinfo_map_.find(data_seq[i].host.hostname);
            //Dealing with hosts table
            if(hostinfo_map_it == hostinfo_map_.end()) {
                hostinfo_map_[data_seq[i].host.hostname] = data_seq[i].host.hostname;
                syslog(LOG_WARNING, "system_monitor, Inserted %s",data_seq[i].host.hostname);
	            query << "REPLACE INTO hosts VALUES (" <<
	                    "'" << data_seq[i].host.hostname << "'," << 
	                    "'" << data_seq[i].host.sysname << "'," <<
	                    "'" << data_seq[i].host.release_name << "'," <<
	                    "'" << data_seq[i].host.machine << "')";
                query.execute();
            }    
            
            		
			//Round robin storage
			if(rowcount_map_[data_seq[i].host.hostname] >= rows_)
				rowcount_map_[data_seq[i].host.hostname] = 0;
				
			query << "REPLACE INTO system_monitor VALUES (" <<
					rowcount_map_[data_seq[i].host.hostname] << "," <<
					"'" << data_seq[i].host.hostname << "'"<< "," <<
					"null" << "," <<
					data_seq[i].cpu_temp << "," <<
					data_seq[i].lavg.load_one << ","  <<
					data_seq[i].lavg.load_five << "," <<
					data_seq[i].lavg.load_fifteen << "," <<
					data_seq[i].cpu_user << "," <<
					data_seq[i].cpu_system << "," <<
					data_seq[i].cpu_idle << "," <<
					data_seq[i].mem_total << "," <<
					data_seq[i].mem_used << "," <<
					data_seq[i].mem_free << ","<<
					data_seq[i].swap_total << "," <<
					data_seq[i].swap_used << "," <<
					data_seq[i].swap_free << ")";

			query.execute();
			rowcount_map_[data_seq[i].host.hostname]++;

		}


	}

	retcode = sysmon_status_reader->return_loan(data_seq, info_seq);
	if (retcode != DDS_RETCODE_OK) {
		syslog(LOG_ERR, "system_monitor, return loan error %d", retcode);
	}
}


